"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.render = render;
exports.canContainsChildren = void 0;

var _dom = _interopRequireDefault(require("sketch/dom"));

var _frame = require("./frame");

var _props = require("./props");

var _styles = require("./styles");

var _symbols = require("./utils/symbols");

var _hooks = require("./hooks");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// eslint-disable-line
const document = _dom.default.getSelectedDocument();

const mountStrategy = {
  artboard(vlayer) {
    const Artboard = _dom.default.Artboard;
    const config = (0, _props.parseConfig)('artboard', vlayer.props);
    const from = config.from;

    if (from && from.type === String(_dom.default.Types.SymbolMaster)) {
      return from.toArtboard();
    }

    return new Artboard(config);
  },

  group(vlayer) {
    const Group = _dom.default.Group;
    const config = (0, _props.parseConfig)('group', vlayer.props);
    return new Group(config);
  },

  text(vlayer) {
    const Text = _dom.default.Text;
    const config = (0, _props.parseConfig)('text', vlayer.props);
    return new Text(config);
  },

  shape(vlayer) {
    const Shape = _dom.default.Shape;
    const config = (0, _props.parseConfig)('shape', vlayer.props);
    return new Shape(config);
  },

  image(vlayer) {
    const Image = _dom.default.Image;
    const config = (0, _props.parseConfig)('image', vlayer.props);
    return new Image(config);
  },

  symbolmaster(vlayer) {
    const SymbolMaster = _dom.default.SymbolMaster;
    const config = (0, _props.parseConfig)('symbolmaster', vlayer.props);
    const from = config.from;

    if (from && from.type === String(_dom.default.Types.Artboard)) {
      return SymbolMaster.fromArtboard(from);
    }

    return new SymbolMaster(config);
  },

  symbolinstance(vlayer) {
    const SymbolInstance = _dom.default.SymbolInstance;
    const config = (0, _props.parseConfig)('symbolinstance', vlayer.props);
    const from = config.from;

    if (from) {
      if (typeof from.url === 'string' && typeof from.path === 'string') {
        return (0, _symbols.getInstanceByPath)(from.url, from.path, from.timestamp);
      }
    }

    return new SymbolInstance(config);
  }

};
const canContainsChildren = {
  artboard: true,
  group: true
};
exports.canContainsChildren = canContainsChildren;

function mountComponent(vlayer, parentLayer) {
  const Component = vlayer.type,
        props = vlayer.props;
  const instance = new Component(props);
  const renderedVlayer = typeof instance.render !== 'function' ? instance : instance.render();
  const layer = mount(renderedVlayer, parentLayer);
  instance.$root = layer;
  (0, _hooks.callHook)(instance, _hooks.hooks.DID_MOUNT, layer);
  return layer;
}

function mountChildren(childrenOrVnode, parentLayer) {
  const typeOfChildren = typeof childrenOrVnode;
  const children = childrenOrVnode;

  if (Array.isArray(children)) {
    children.forEach(childVlayer => {
      const typeOfChild = typeof childVlayer;

      if (Array.isArray(childVlayer)) {
        mountChildren(childVlayer, parentLayer);
      } else if (typeOfChild === 'object') {
        const typeOfChildType = typeof childVlayer.type;

        if (typeOfChildType === 'function') {
          mountComponent(childVlayer, parentLayer);
        } else if (typeOfChildType === 'string') {
          mount(childVlayer, parentLayer);
        }
      }
    });
  } else if (typeOfChildren === 'object') {
    const typeOfChildrenType = typeof children.type;

    if (typeOfChildrenType === 'string') {
      mount(children, parentLayer);
    } else if (typeOfChildrenType === 'function') {
      mountComponent(children, parentLayer);
    }
  }

  return children;
}

function mountLayer(vlayer) {
  const type = vlayer.type; // TODO: throw error if is not a function

  const layer = mountStrategy[type](vlayer);
  return layer;
}

function mount(vlayer, parentLayer) {
  const type = vlayer.type,
        props = vlayer.props,
        frame = vlayer.frame,
        style = vlayer.style;
  const typeOfType = typeof type;
  let layer;

  if (typeOfType === 'function') {
    layer = mountComponent(vlayer, parentLayer);
  } else if (typeOfType === 'string' || typeOfType === 'number') {
    layer = mountLayer(vlayer);
  }

  layer.parent = parentLayer;
  const children = props.children;

  if (typeOfType !== 'function' && // TODO: add warning if not can contains children
  canContainsChildren[type] && typeof children === 'object' && children !== null) {
    props.children = mountChildren(children, layer);
  }

  if (props.adjustToFit && typeof layer.adjustToFit === 'function') {
    layer.adjustToFit();
  }

  (0, _frame.initFrame)(layer, frame);
  (0, _props.mapProps)(layer, props);
  (0, _styles.mapStyles)(layer, style);
  return layer;
}

function render(vlayer, parentLayer = document.selectedPage) {
  parentLayer.layers = [];
  const layer = mount(vlayer, parentLayer);
  return layer;
}